@@ -34,6 +34,8 @@
 
 namespace KDevelop
 {
+class ControlFlowGraph;
+class DataAccessRepository;
 class BackgroundParser;
 class TopDUContext;
 class ReferencedTopDUContext;
@@ -48,12 +50,17 @@ class KDEVPLATFORMLANGUAGE_EXPORT ParseJob : public ThreadWeaver::JobSequence
     Q_OBJECT
 
 public:
-    ParseJob( const KUrl &url );
+    explicit ParseJob( const IndexedString &url, ILanguageSupport* languageSupport );
     /**
      * _No_ mutexes/locks are allowed to be locked when this object is destroyed (except for optionally the foreground lock)
      * */
     virtual ~ParseJob();
 
+    /**
+     * @return the language support that created this parse job.
+     */
+    ILanguageSupport* languageSupport() const;
+
     struct Contents {
         // Modification-time of the read content
         ModificationRevision modification;
@@ -79,30 +86,30 @@ public:
 
     /**
      * _No_ mutexes/locks are allowed to be locked when this is called (except for optionally the foreground lock)
-     * 
+     *
      * Locks the document revision so that mapping from/to the revision in the editor using MovingInterface will be possible.
-     * 
+     *
      * Returns an invalid pointer if the call succeeds, and a valid one if the reading fails.
      * */
     KDevelop::ProblemPointer readContents();
-    
+
     /**
      * After reading the contents, you can call this to retrieve it.
      * */
     const Contents& contents() const;
-    
+
     /**
      * Translates the given context from its previous revision to the revision that has
      * been retrieved during readContents(). The top-context meta-data will be updated
      * with the revision.
-     * 
+     *
      * This can be done after reading the context before updating, so
      * that the correct ranges are matched onto each other during the update.
-     * 
+     *
      * _No_ mutexes/locks are allowed to be locked when this is called (except for optionally the foreground lock)
      */
     void translateDUChainToRevision(TopDUContext* context);
-    
+
     /**
      * Query whether this job is needed to be waited for when trying to process a job with a lower priority.
      **/
@@ -124,8 +131,8 @@ public:
     * The notification is guaranteed to be called once the parse-job finishes, from within its destructor.
     * The given top-context may be invalid if the update failed.
     */
-    Q_SCRIPTABLE void setNotifyWhenReady(QList<QWeakPointer<QObject> > notify);
-    
+    Q_SCRIPTABLE void setNotifyWhenReady(const QList<QWeakPointer<QObject> >& notify);
+
     /// Sets the du-context that was created by this parse-job
     Q_SCRIPTABLE virtual void setDuChain(ReferencedTopDUContext duChain);
     /// Returns the set du-context, or zero of none was set.
@@ -143,31 +150,66 @@ public:
 
     /// Set the minimum features the resulting top-context should have
     Q_SCRIPTABLE void setMinimumFeatures(TopDUContext::Features features);
-    
+
     /// Minimum set of features the resulting top-context should have
     Q_SCRIPTABLE TopDUContext::Features minimumFeatures() const;
-    
+
     /// Allows statically specifying an amount of features required for an url.
     /// These features will automatically be or'ed with the minimumFeatures() returned
     /// by any ParseJob with the given url.
     /// Since this causes some additional complixity in update-checking, minimum features should not
     /// be set permanently.
-    static void setStaticMinimumFeatures(IndexedString url, TopDUContext::Features features);
-    
+    static void setStaticMinimumFeatures(const IndexedString& url, TopDUContext::Features features);
+
     /// Must be called exactly once for each call to setStaticMinimumFeatures, with the same features.
-    static void unsetStaticMinimumFeatures(IndexedString url, TopDUContext::Features features);
-    
+    static void unsetStaticMinimumFeatures(const IndexedString& url, TopDUContext::Features features);
+
     /// Returns the statically set minimum features for  the given url, or zero.
-    static TopDUContext::Features staticMinimumFeatures(IndexedString url);
-    
+    static TopDUContext::Features staticMinimumFeatures(const IndexedString& url);
+
     /// Returns whether there is minimum features set up for some url
     static bool hasStaticMinimumFeatures();
+    
+    ///Returns a structure containing information about data accesses in the parsed file.
+    /// It's up to the caller to remove the returned instance
+    virtual KDevelop::DataAccessRepository* dataAccessInformation();
+    
+    ///Returns a control flow graph for the code in the parsed file.
+    /// It's up to the caller to remove the returned instance
+    virtual KDevelop::ControlFlowGraph* controlFlowGraph();
 
 Q_SIGNALS:
     /**Can be used to give progress feedback to the background-parser. @param value should be between 0 and 1, where 0 = 0% and 1 = 100%
      * @param text may be a text that describes the current state of parsing
      * Do not trigger this too often, for performance reasons. */
-    void progress(KDevelop::ParseJob*, float value, QString text);
+    void progress(KDevelop::ParseJob*, float value, const QString& text);
+
+protected:
+    /**
+     * Checks whether there is already an up to date context available for the
+     * current document. If so, it returns true and ensures that the document
+     * is highlighted properly. Otherwise returns false.
+     *
+     * NOTE: This should be called while holding an URLParseLock for the
+     * current document.
+     *
+     * @param languageString The unique string identifying your language.
+     * This must be the same as you assign to the DUChain's environment file.
+     *
+     * @return True if an update is required, false if the job can return early.
+     */
+    bool isUpdateRequired(const IndexedString& languageString);
+
+    /**
+     * Trigger an update to the code highlighting of the current file based
+     * on the DUChain set in setDuChain.
+     *
+     * If the file for this parse job is not opened in an editor or if the language
+     * support does not return a code highlighter, this will do nothing.
+     *
+     * NOTE: No DUChain lock should be held when you call this.
+     */
+    void highlightDUChain();
 
 private:
     class ParseJobPrivate* const d;
