@@ -77,7 +77,7 @@ public:
       //The thread-safety is given because all threads will have the same result, and it will only be written once at the end.
       uint hash = m_identifier.hash();
       FOREACH_FUNCTION_STATIC(const IndexedTypeIdentifier& templateIdentifier, templateIdentifiers)
-        hash = hash * 13 + IndexedTypeIdentifier(templateIdentifier).hash();
+        hash = hash * 13 + templateIdentifier.hash();
       hash += m_unique;
       m_hash = hash;
     }
@@ -1048,7 +1048,9 @@ QString IndexedTypeIdentifier::toString(bool ignoreExplicitlyGlobal) const {
       ret += "const";
   }
 
-  if(isReference())
+  if(isRValue())
+    ret += "&&";
+  else if(isReference())
     ret += '&';
   return ret;
 }
@@ -1272,4 +1274,16 @@ void initIdentifierRepository() {
 
 }
 
+QDebug operator<<(QDebug s, const KDevelop::Identifier& identifier)
+{
+  s.nospace() << identifier.toString();
+  return s.space();
+}
+
+QDebug operator<<(QDebug s, const KDevelop::QualifiedIdentifier& identifier)
+{
+  s.nospace() << identifier.toString();
+  return s.space();
+}
+
 // kate: space-indent on; indent-width 2; tab-width 4; replace-tabs on; auto-insert-doxygen on
