@@ -21,29 +21,44 @@
 
 #include "contextbrowser.h"
 
+#include "contextbrowserview.h"
 #include "browsemanager.h"
 
-///TODO: cleanup includes
+///TODO: remove unneeded includes
+#include <memory>
+#include <cstdlib>
+
 #include <QTimer>
 #include <QApplication>
+#include <QToolButton>
+#include <QLayout>
 #include <qalgorithms.h>
-#include <klocale.h>
-#include <kaction.h>
-#include <kpluginfactory.h>
-#include <kpluginloader.h>
-#include <ktexteditor/view.h>
-#include <ktexteditor/document.h>
+
+#include <KLocale>
+#include <KAction>
+#include <KPluginFactory>
+#include <KPluginLoader>
+#include <KActionCollection>
+#include <KAboutData>
+#include <KDebug>
+
+#include <KTextEditor/View>
+#include <KTextEditor/Document>
 #include <KTextEditor/TextHintInterface>
-#include <kactioncollection.h>
-#include <kaboutdata.h>
-#include <kdebug.h>
+#include <KTextEditor/CodeCompletionInterface>
+
 #include <interfaces/icore.h>
 #include <interfaces/idocumentcontroller.h>
 #include <interfaces/ilanguage.h>
 #include <interfaces/iuicontroller.h>
 #include <interfaces/ilanguagecontroller.h>
 #include <interfaces/contextmenuextension.h>
+#include <interfaces/iplugincontroller.h>
+
 #include <language/interfaces/codecontext.h>
+#include <language/interfaces/ilanguagesupport.h>
+#include <language/interfaces/iquickopen.h>
+
 #include <language/duchain/duchainlock.h>
 #include <language/duchain/duchain.h>
 #include <language/duchain/ducontext.h>
@@ -53,23 +68,19 @@
 #include <language/duchain/dumpchain.h>
 #include <language/duchain/functiondefinition.h>
 #include <language/duchain/parsingenvironment.h>
-#include <language/interfaces/ilanguagesupport.h>
-#include <language/backgroundparser/backgroundparser.h>
-#include <language/backgroundparser/parsejob.h>
-#include "contextbrowserview.h"
 #include <language/duchain/uses.h>
 #include <language/duchain/specializationstore.h>
 #include <language/duchain/aliasdeclaration.h>
-#include <language/util/navigationtooltip.h>
-#include <language/duchain/navigation/abstractnavigationwidget.h>
-#include <language/interfaces/iquickopen.h>
-#include <interfaces/iplugincontroller.h>
-#include <sublime/mainwindow.h>
-#include <memory>
-#include <QToolButton>
-#include <QLayout>
 #include <language/duchain/types/functiontype.h>
-#include <ktexteditor/codecompletioninterface.h>
+
+#include <language/duchain/navigation/abstractnavigationwidget.h>
+
+#include <language/backgroundparser/backgroundparser.h>
+#include <language/backgroundparser/parsejob.h>
+
+#include <language/util/navigationtooltip.h>
+
+#include <sublime/mainwindow.h>
 
 static const unsigned int highlightingTimeout = 150;
 static const float highlightingZDepth = -5000;
@@ -243,6 +254,8 @@ void ContextBrowserPlugin::createActionsForMainWindow(Sublime::MainWindow* windo
     QWidget* w = toolbarWidgetForMainWindow(window);
     w->setHidden(false);
     outline->setDefaultWidget(w);
+    // Add to the actioncollection so one can set global shortcuts for the action
+    actions.addAction("find_uses", m_findUses);
 }
 
 void ContextBrowserPlugin::nextContextShortcut()
@@ -265,6 +278,8 @@ ContextBrowserPlugin::ContextBrowserPlugin(QObject *parent, const QVariantList&)
     , m_viewFactory(new ContextBrowserViewFactory(this))
     , m_nextHistoryIndex(0)
 {
+  KDEV_USE_EXTENSION_INTERFACE( IContextBrowser )
+
   core()->uiController()->addToolView(i18n("Code Browser"), m_viewFactory);
 
   connect( core()->documentController(), SIGNAL(textDocumentCreated(KDevelop::IDocument*)), this, SLOT(textDocumentCreated(KDevelop::IDocument*)) );
@@ -278,7 +293,7 @@ ContextBrowserPlugin::ContextBrowserPlugin(QObject *parent, const QVariantList&)
   connect( m_updateTimer, SIGNAL(timeout()), this, SLOT(updateViews()) );
   
   //Needed global action for the context-menu extensions
-  m_findUses = new QAction(i18n("Find Uses"), this);
+  m_findUses = new KAction(i18n("Find Uses"), this);
   connect(m_findUses, SIGNAL(triggered(bool)), this, SLOT(findUses()));
 }
 
@@ -322,25 +337,47 @@ KDevelop::ContextMenuExtension ContextBrowserPlugin::contextMenuExtension(KDevel
   return menuExt;
 }
 
+void ContextBrowserPlugin::showUses(const DeclarationPointer& declaration)
+{
+  QMetaObject::invokeMethod(this, "showUsesDelayed", Qt::QueuedConnection,
+                            Q_ARG(KDevelop::DeclarationPointer, declaration));
+}
+
+void ContextBrowserPlugin::showUsesDelayed(const DeclarationPointer& declaration)
+{
+  DUChainReadLocker lock;
+
+  Declaration* decl = declaration.data();
+  if(!decl) {
+    return;
+  }
+  QWidget* toolView = ICore::self()->uiController()->findToolView(i18n("Code Browser"), m_viewFactory, KDevelop::IUiController::CreateAndRaise);
+  if(!toolView) {
+    return;
+  }
+  ContextBrowserView* view = dynamic_cast<ContextBrowserView*>(toolView);
+  Q_ASSERT(view);
+  view->allowLockedUpdate();
+  view->setDeclaration(decl, decl->topContext(), true);
+  //We may get deleted while the call to acceptLink, so make sure we don't crash in that case
+  QPointer<AbstractNavigationWidget> widget = dynamic_cast<AbstractNavigationWidget*>(view->navigationWidget());
+  if(widget && widget->context()) {
+    NavigationContextPointer nextContext = widget->context()->execute(
+      NavigationAction(declaration, KDevelop::NavigationAction::ShowUses));
+
+    if(widget) {
+      widget->setContext( nextContext );
+    }
+  }
+}
+
 void ContextBrowserPlugin::findUses()
 {
   QAction* action = qobject_cast<QAction*>(sender());
   Q_ASSERT(action);
-  DUChainReadLocker lock(DUChain::lock());
   
   KDevelop::IndexedDeclaration decl = action->data().value<KDevelop::IndexedDeclaration>();
-  if(!decl.data())
-    return;
-  QWidget* widget = ICore::self()->uiController()->findToolView(i18n("Code Browser"), m_viewFactory, KDevelop::IUiController::CreateAndRaise);
-  if(!widget)
-    return;
-  ContextBrowserView* view = dynamic_cast<ContextBrowserView*>(widget);
-  Q_ASSERT(view);
-  view->allowLockedUpdate();
-  view->setDeclaration(decl.data(), decl.data()->topContext(), true);
-  KDevelop::AbstractNavigationWidget* navigationWidget = dynamic_cast<KDevelop::AbstractNavigationWidget*>(view->navigationWidget());
-  if(navigationWidget)
-    navigationWidget->executeContextAction("show_uses");
+  showUses(DeclarationPointer(decl.declaration()));
 }
 
 void ContextBrowserPlugin::textHintRequested(const KTextEditor::Cursor& cursor, QString&) {
@@ -380,6 +417,28 @@ void ContextBrowserPlugin::hideTooTip() {
   }
 }
 
+// TODO: this is a hack, but Kate does not provide interface for this
+static int getLineHeight(KTextEditor::View* view, int curLine)
+{
+  KTextEditor::Cursor c(curLine, 0);
+  int currentHeight = view->cursorToCoordinate(c).y();
+  c.setLine(curLine + 1);
+  if (view->cursorToCoordinate(c).y() < 0) {
+    c.setLine(curLine - 1);
+  }
+  return std::abs(view->cursorToCoordinate(c).y() - currentHeight);
+}
+
+static QRect getItemBoundingRect(const KUrl& viewUrl, KTextEditor::View* view, KTextEditor::Cursor itemPosition)
+{
+  DUChainReadLocker lock;
+  KTextEditor::Range itemRange = DUChainUtils::itemRangeUnderCursor(viewUrl, SimpleCursor(itemPosition));
+  QPoint startPoint = view->mapToGlobal(view->cursorToCoordinate(itemRange.start()));
+  QPoint endPoint = view->mapToGlobal(view->cursorToCoordinate(itemRange.end()));
+  endPoint.ry() += getLineHeight(view, itemPosition.line());
+  return QRect(startPoint, endPoint);
+}
+
 void ContextBrowserPlugin::showToolTip(KTextEditor::View* view, KTextEditor::Cursor position) {
   
   ContextBrowserView* contextView = browserViewForWidget(view);
@@ -429,6 +488,7 @@ void ContextBrowserPlugin::showToolTip(KTextEditor::View* view, KTextEditor::Cur
     }
     
     KDevelop::NavigationToolTip* tooltip = new KDevelop::NavigationToolTip(view, view->mapToGlobal(view->cursorToCoordinate(position)) + QPoint(20, 40), navigationWidget);
+    tooltip->addExtendRect(getItemBoundingRect(viewUrl, view, position));
     tooltip->resize( navigationWidget->sizeHint() + QSize(10, 10) );
     kDebug() << "tooltip size" << tooltip->size();
     m_currentToolTip = tooltip;
@@ -1302,6 +1362,4 @@ void ContextBrowserPlugin::HistoryEntry::setCursorPosition(const KDevelop::Simpl
     }
 }
 
-#include "contextbrowser.moc"
-
 // kate: space-indent on; indent-width 2; tab-width 4; replace-tabs on; auto-insert-doxygen on
