@@ -50,6 +50,7 @@
 
 #include <KCModuleProxy>
 #include <KConfigDialog>
+#include <KShell>
 #include <kcmoduleinfo.h>
 
 KIcon NativeAppConfigPage::icon() const
@@ -113,13 +114,13 @@ NativeAppConfigPage::NativeAppConfigPage( QWidget* parent )
     removeDependency->setIcon( KIcon("list-remove") );
     moveDepUp->setIcon( KIcon("go-up") );
     moveDepDown->setIcon( KIcon("go-down") );
+    browseProject->setIcon(KIcon("folder-document"));
     
     //Set workingdirectory widget to ask for directories rather than files
     workingDirectory->setMode(KFile::Directory | KFile::ExistingOnly | KFile::LocalOnly);
 
     KDevelop::EnvironmentGroupList env( KGlobal::config() );
     environment->addItems( env.groups() );
-    browseProject->setIcon(KIcon("folder-document"));
 
     configureEnvironment->setSelectionWidget(environment);
 
@@ -156,7 +157,6 @@ void NativeAppConfigPage::depEdited( const QString& str )
 {
     int pos;
     QString tmp = str;
-    kDebug() << str << targetDependency->validator();
     addDependency->setEnabled( !str.isEmpty() 
                                && ( !targetDependency->validator() 
                                || targetDependency->validator()->validate( tmp, pos ) == QValidator::Acceptable ) );
@@ -221,9 +221,8 @@ void NativeAppConfigPage::moveDependencyUp()
 
 void NativeAppConfigPage::addDep()
 {
-    KDevelop::ProjectModel* model = KDevelop::ICore::self()->projectController()->projectModel();
     KIcon icon;
-    KDevelop::ProjectBaseItem* pitem = itemForPath(KDevelop::splitWithEscaping(targetDependency->text(),'/', '\\'), model);
+    KDevelop::ProjectBaseItem* pitem = targetDependency->currentItem();
     if(pitem)
         icon= KIcon(pitem->iconName());
 
@@ -350,7 +349,7 @@ NativeAppConfigType::~NativeAppConfigType()
 
 QString NativeAppConfigType::name() const
 {
-    return i18n("Native Application");
+    return i18n("Application");
 }
 
 
@@ -366,7 +365,7 @@ QString NativeAppConfigType::id() const
 
 KIcon NativeAppConfigType::icon() const
 {
-    return KIcon("system-run");
+    return KIcon("application-x-executable");
 }
 
 bool NativeAppConfigType::canLaunch ( KDevelop::ProjectBaseItem* item ) const
@@ -397,7 +396,7 @@ void NativeAppConfigType::configureLaunchFromCmdLineArguments ( KConfigGroup cfg
     cfg.writeEntry( ExecutePlugin::executableEntry, args.first() );
     QStringList a(args);
     a.removeFirst();
-    cfg.writeEntry( ExecutePlugin::argumentsEntry, a);
+    cfg.writeEntry( ExecutePlugin::argumentsEntry, KShell::joinArgs(a) );
     cfg.sync();
 }
 
@@ -426,9 +425,11 @@ QMenu* NativeAppConfigType::launcherSuggestions()
             QMenu* projectMenu = ret->addMenu(project->name());
             foreach(KDevelop::ProjectTargetItem* target, targets) {
                 if(target->executable()) {
-                    QString path = KDevelop::joinWithEscaping(model->pathFromIndex(target->index()),'/','\\');
-                    QAction* act = projectMenu->addAction(path);
-                    act->setData(path);
+                    QStringList path = model->pathFromIndex(target->index());
+                    QAction* act = projectMenu->addAction(QString());
+                    act->setData(KDevelop::joinWithEscaping(path, '/','\\'));
+                    path.removeFirst();
+                    act->setText(path.join("/"));
                     connect(act, SIGNAL(triggered(bool)), SLOT(suggestionTriggered()));
                 }
             }
