@@ -30,7 +30,6 @@
 #include <QFileInfo>
 #include <QDir>
 #include <QDateTime>
-#include <QtGui/QTextEdit>
 
 #include <interfaces/icore.h>
 #include <interfaces/iprojectcontroller.h>
@@ -43,6 +42,7 @@
 #include <vcs/vcsannotation.h>
 #include <vcs/widgets/standardvcslocationwidget.h>
 #include <KIO/CopyJob>
+#include <KIO/NetAccess>
 #include "gitclonejob.h"
 #include <interfaces/contextmenuextension.h>
 #include <QMenu>
@@ -50,6 +50,7 @@
 #include "stashmanagerdialog.h"
 #include <KMessageBox>
 #include <KStandardDirs>
+#include <KTextEdit>
 #include "gitjob.h"
 #include "gitmessagehighlighter.h"
 
@@ -58,6 +59,16 @@ K_EXPORT_PLUGIN(KDevGitFactory(KAboutData("kdevgit","kdevgit",ki18n("Git"),"0.1"
 
 using namespace KDevelop;
 
+QVariant runSynchronously(KDevelop::VcsJob* job)
+{
+    QVariant ret;
+    if(job->exec() && job->status()==KDevelop::VcsJob::JobSucceeded) {
+        ret = job->fetchResults();
+    }
+    delete job;
+    return ret;
+}
+
 namespace
 {
     
@@ -172,6 +183,7 @@ GitPlugin::GitPlugin( QObject *parent, const QVariantList & )
 
     KDEV_USE_EXTENSION_INTERFACE( KDevelop::IBasicVersionControl )
     KDEV_USE_EXTENSION_INTERFACE( KDevelop::IDistributedVersionControl )
+    KDEV_USE_EXTENSION_INTERFACE( KDevelop::IBranchingVersionControl )
 
     m_hasError = false;
     core()->uiController()->addToolView(i18n("Git"), dvcsViewFactory());
@@ -270,6 +282,9 @@ bool GitPlugin::isValidDirectory(const KUrl & dirPath)
 bool GitPlugin::isVersionControlled(const KUrl &path)
 {
     QFileInfo fsObject(path.toLocalFile());
+    if (!fsObject.exists()) {
+        return false;
+    }
     if (fsObject.isDir()) {
         return isValidDirectory(path);
     }
@@ -410,22 +425,65 @@ void GitPlugin::addNotVersionedFiles(const QDir& dir, const KUrl::List& files)
     }
 }
 
+bool isEmptyDirStructure(const QDir &dir)
+{
+    foreach (const QFileInfo &i, dir.entryInfoList(QDir::AllEntries | QDir::NoDotAndDotDot)) {
+        if (i.isDir()) {
+            if (!isEmptyDirStructure(QDir(i.filePath()))) return false;
+        } else if (i.isFile()) {
+            return false;
+        }
+    }
+    return true;
+}
+
 VcsJob* GitPlugin::remove(const KUrl::List& files)
 {
     if (files.isEmpty())
         return errorsFound(i18n("No files to remove"));
-    QDir dir = dotGitDirectory(files.front());
+    QDir dotGitDir = dotGitDirectory(files.front());
 
-    QStringList otherStr = getLsFiles(dir, QStringList() << "--others" << "--" << files.front().toLocalFile(), KDevelop::OutputJob::Silent);
-    if(otherStr.isEmpty()) {
-        DVcsJob* job = new GitJob(dir, this);
-        job->setType(VcsJob::Remove);
-        *job << "git" << "rm" << "-r";
-        *job << "--" << files;
-        return job;
-    } else {
-        return new StandardJob(this, KIO::trash(files), KDevelop::OutputJob::Silent);
+
+    KUrl::List files_(files);
+
+    QMutableListIterator<KUrl> i(files_);
+    while (i.hasNext()) {
+        KUrl file = i.next();
+        QFileInfo fileInfo(file.toLocalFile());
+
+        QStringList otherStr = getLsFiles(dotGitDir, QStringList() << "--others" << "--" << file.toLocalFile(), KDevelop::OutputJob::Silent);
+        kDebug() << "other files" << otherStr;
+        if(!otherStr.isEmpty()) {
+            //remove files not under version control
+            KUrl::List otherFiles;
+            foreach(const QString &f, otherStr) {
+                otherFiles << KUrl::fromLocalFile(dotGitDir.path()+'/'+f);
+            }
+            if (fileInfo.isFile()) {
+                //if it's an unversioned file we are done, don't use git rm on it
+                i.remove();
+            }
+            KIO::NetAccess::synchronousRun(KIO::trash(otherFiles), 0);
+        }
+
+        if (fileInfo.isDir()) {
+            if (isEmptyDirStructure(QDir(file.toLocalFile()))) {
+                //remove empty folders, git doesn't do that
+                kDebug() << "empty folder, removing" << file;
+                KIO::NetAccess::synchronousRun(KIO::trash(file), 0);
+                //we already deleted it, don't use git rm on it
+                i.remove();
+            }
+        }
     }
+
+    if (files_.isEmpty()) return 0;
+
+    DVcsJob* job = new GitJob(dotGitDir, this);
+    job->setType(VcsJob::Remove);
+    *job << "git" << "rm" << "-r";
+    *job << "--" << files_;
+    return job;
 }
 
 VcsJob* GitPlugin::log(const KUrl& localLocation,
@@ -822,11 +880,11 @@ QList<DVcsEvent> GitPlugin::getAllCommits(const QString &repo)
 
 void GitPlugin::initBranchHash(const QString &repo)
 {
-    QStringList branches = listBranches(KUrl(repo));
-    kDebug() << "BRANCHES: " << branches;
+    QStringList gitBranches = runSynchronously(branches(KUrl(repo))).toStringList();
+    kDebug() << "BRANCHES: " << gitBranches;
     //Now root branch is the current branch. In future it should be the longest branch
     //other commitLists are got with git-rev-lits branch ^br1 ^ br2
-    QString root = GitPlugin::curBranch(repo);
+    QString root = runSynchronously(currentBranch(repo)).toString();
     QScopedPointer<DVcsJob> job(gitRevList(repo, QStringList(root)));
     bool ret = job->exec();
     Q_ASSERT(ret && job->status()==VcsJob::JobSucceeded && "TODO: provide a fall back in case of failing");
@@ -834,12 +892,12 @@ void GitPlugin::initBranchHash(const QString &repo)
     QStringList commits = job->output().split('\n', QString::SkipEmptyParts);
 //     kDebug() << "\n\n\n commits" << commits << "\n\n\n";
     branchesShas.append(commits);
-    foreach(const QString &branch, branches)
+    foreach(const QString &branch, gitBranches)
     {
         if (branch == root)
             continue;
         QStringList args(branch);
-        foreach(const QString &branch_arg, branches)
+        foreach(const QString &branch_arg, gitBranches)
         {
             if (branch_arg != branch)
                 //man gitRevList for '^'
@@ -1006,28 +1064,38 @@ void GitPlugin::parseGitStatusOutput_old(DVcsJob* job)
 void GitPlugin::parseGitStatusOutput(DVcsJob* job)
 {
     QStringList outputLines = job->output().split('\n', QString::SkipEmptyParts);
-    const KUrl workingDir = job->directory().absolutePath();
-    const KUrl dotGit = dotGitDirectory(workingDir).absolutePath();
+    KUrl workingDir = job->directory().absolutePath();
+    KUrl dotGit = dotGitDirectory(workingDir).absolutePath();;
+    dotGit.adjustPath(KUrl::AddTrailingSlash);
+    workingDir.adjustPath(KUrl::AddTrailingSlash);
     
     QVariantList statuses;
-    QList<KUrl> processedFiles;
+    KUrl::List processedFiles;
     
     foreach(const QString& line, outputLines) {
         //every line is 2 chars for the status, 1 space then the file desc
         QString curr=line.right(line.size()-3);
         QString state = line.left(2);
         
-        int arrow = curr.indexOf("-> ");
-        if(arrow>=0)
-            curr = curr.right(curr.size()-arrow-3);
+        int arrow = curr.indexOf(" -> ");
+        if(arrow>=0) {
+            VcsStatusInfo status;
+            status.setUrl(KUrl(dotGit, curr.left(arrow)));
+            status.setState(VcsStatusInfo::ItemDeleted);
+            statuses.append(qVariantFromValue<VcsStatusInfo>(status));
+            processedFiles += status.url();
+            
+            curr = curr.mid(arrow+4);
+        }
         
-        KUrl fileUrl = dotGit;
-        fileUrl.addPath(curr);
-        processedFiles.append(fileUrl);
+        if(curr.startsWith('\"') && curr.endsWith('\"')) { //if the path is quoted, unquote
+            curr = curr.mid(1, curr.size()-2);
+        }
         
         VcsStatusInfo status;
-        status.setUrl(fileUrl);
+        status.setUrl(KUrl(dotGit, curr));
         status.setState(messageToState(state));
+        processedFiles.append(status.url());
         
         kDebug() << "Checking git status for " << line << curr << messageToState(state);
         
@@ -1042,8 +1110,7 @@ void GitPlugin::parseGitStatusOutput(DVcsJob* job)
     //here we add the already up to date files
     QStringList files = getLsFiles(job->directory(), QStringList() << "-c" << "--" << paths, OutputJob::Silent);
     foreach(const QString& file, files) {
-        KUrl fileUrl = workingDir;
-        fileUrl.addPath(file);
+        KUrl fileUrl(workingDir, file);
         
         if(!processedFiles.contains(fileUrl)) {
             VcsStatusInfo status;
@@ -1094,9 +1161,9 @@ DVcsJob* GitPlugin::gitRevParse(const QString &repository, const QStringList &ar
     return job;
 }
 
-DVcsJob* GitPlugin::gitRevList(const QString &repository, const QStringList &args)
+DVcsJob* GitPlugin::gitRevList(const QString& directory, const QStringList& args)
 {
-    DVcsJob* job = new DVcsJob(QDir(repository), this, KDevelop::OutputJob::Silent);
+    DVcsJob* job = new DVcsJob(urlDir(directory), this, KDevelop::OutputJob::Silent);
     {
         *job << "git" << "rev-list" << args;
         return job;
@@ -1232,7 +1299,7 @@ VcsJob* GitPlugin::update(const KUrl::List& localLocations, const KDevelop::VcsR
     }
 }
 
-void GitPlugin::setupCommitMessageEditor(const KUrl& localLocation, QTextEdit* editor) const
+void GitPlugin::setupCommitMessageEditor(const KUrl& localLocation, KTextEdit* editor) const
 {
     new GitMessageHighlighter(editor);
     QFile mergeMsgFile(dotGitDirectory(localLocation).filePath(".git/MERGE_MSG"));
@@ -1240,10 +1307,10 @@ void GitPlugin::setupCommitMessageEditor(const KUrl& localLocation, QTextEdit* e
     // the memory. 1Mb seems to be good value since it's rather strange to have so huge commit
     // message.
     static const qint64 maxMergeMsgFileSize = 1024*1024;
-    if (!mergeMsgFile.exists() || mergeMsgFile.size() > maxMergeMsgFileSize)
+    if (mergeMsgFile.size() > maxMergeMsgFileSize || !mergeMsgFile.open(QIODevice::ReadOnly))
         return;
-    mergeMsgFile.open(QIODevice::ReadOnly);
-    QString mergeMsg = QString::fromLocal8Bit(mergeMsgFile.readAll());
+    
+    QString mergeMsg = QString::fromLocal8Bit(mergeMsgFile.read(maxMergeMsgFileSize));
     editor->setPlainText(mergeMsg);
 }
 
