@@ -70,6 +70,7 @@ Q_DECLARE_METATYPE( const Diff2::DiffModel* )
 
 void PatchReviewPlugin::seekHunk( bool forwards, const KUrl& fileName ) {
     try {
+        kDebug() << forwards << fileName << fileName.isEmpty();
         if ( !m_modelList.get() )
             throw "no model";
 
@@ -82,11 +83,10 @@ void PatchReviewPlugin::seekHunk( bool forwards, const KUrl& fileName ) {
 
             if ( !fileName.isEmpty() && fileName != file )
                 continue;
-
+            
             IDocument* doc = ICore::self()->documentController()->documentForUrl( file );
-
-            if ( doc && doc == ICore::self()->documentController()->activeDocument() && m_highlighters.contains( doc->url() ) && m_highlighters[doc->url()] ) {
-                ICore::self()->documentController()->activateDocument( doc );
+            
+            if ( doc && m_highlighters.contains( doc->url() ) && m_highlighters[doc->url()] ) {
                 if ( doc->textDocument() ) {
                     const QList<KTextEditor::MovingRange*> ranges = m_highlighters[doc->url()]->ranges();
 
@@ -108,7 +108,7 @@ void PatchReviewPlugin::seekHunk( bool forwards, const KUrl& fileName ) {
                         if ( bestLine != -1 ) {
                             v->setCursorPosition( KTextEditor::Cursor( bestLine, 0 ) );
                             return;
-                        } else {
+                        } else if(fileName.isEmpty()) {
                             int next = qBound(0, forwards ? a+1 : a-1, m_modelList->modelCount()-1);
                             ICore::self()->documentController()->openDocument(urlForFileModel(m_modelList->modelAt(next)));
                         }
@@ -201,10 +201,6 @@ void PatchReviewPlugin::notifyPatchChanged() {
     m_updateKompareTimer->start( 500 );
 }
 
-void PatchReviewPlugin::showPatch() {
-    startReview( m_patch, OpenAndRaise );
-}
-
 void PatchReviewPlugin::forceUpdate() {
     if( m_patch ) {
         m_patch->update();
@@ -227,20 +223,18 @@ void PatchReviewPlugin::updateKompareModel() {
         m_modelList.reset( 0 );
         delete m_diffSettings;
 
-        emit patchChanged();
-
         {
             IDocument* patchDoc = ICore::self()->documentController()->documentForUrl( m_patch->file() );
             if( patchDoc )
                 patchDoc->reload();
         }
         QString patchFile;
-        if(m_patch->file().isLocalFile())
-          patchFile = m_patch->file().toLocalFile();
-        else if(m_patch->file().isValid() && !m_patch->file().isEmpty()){
-          bool ret = KIO::NetAccess::download(m_patch->file(), patchFile, ICore::self()->uiController()->activeMainWindow());
-          if(!ret)
-            kWarning() << "Problem while downloading: " << m_patch->file();
+        if( m_patch->file().isLocalFile() )
+            patchFile = m_patch->file().toLocalFile();
+        else if( m_patch->file().isValid() && !m_patch->file().isEmpty() ) {
+            bool ret = KIO::NetAccess::download( m_patch->file(), patchFile, ICore::self()->uiController()->activeMainWindow() );
+            if( !ret )
+                kWarning() << "Problem while downloading: " << m_patch->file();
         }
 
         m_diffSettings = new DiffSettings( 0 );
@@ -254,14 +248,7 @@ void PatchReviewPlugin::updateKompareModel() {
         m_modelList->slotKompareInfo( m_kompareInfo.get() );
 
         try {
-            if ( !m_modelList->openDirAndDiff() )
-            {
-#if 0
-                // Don't error out on empty files, as those are valid diffs too
-                if( QFileInfo( m_patch->file().toLocalFile() ).size() != 0 )
-                    throw "could not open diff " + m_patch->file().prettyUrl() + " on " + m_patch->baseDir().prettyUrl();
-#endif
-            }
+            m_modelList->openDirAndDiff();
         } catch ( const QString & str ) {
             throw;
         } catch ( ... ) {
@@ -344,43 +331,6 @@ void PatchReviewPlugin::clearPatch( QObject* _patch ) {
     }
 }
 
-#if 0
-#ifdef HAVE_KOMPARE
-void showDiff( const KDevelop::VcsDiff& d ) {
-    ICore::self()->uiController()->switchToArea( "review", KDevelop::IUiController::ThisWindow );
-    foreach( const VcsLocation& l, d.leftTexts().keys() )
-    {
-        KUrl to;
-        if( d.rightTexts().contains( l ) )
-        {
-            KTemporaryFile temp2;
-            temp2.setSuffix( "2.patch" );
-            //FIXME: don't leak
-            temp2.setAutoRemove( false );
-            temp2.open();
-            QTextStream t2( &temp2 );
-            t2 << d.rightTexts()[l];
-            temp2.close();
-            to = temp2.fileName();
-        }
-        else
-            to = l.localUrl();
-
-        KUrl fakeUrl( to );
-        fakeUrl.setScheme( "kdevpatch" );
-
-        IDocumentFactory* docf = ICore::self()->documentController()->factory( "text/x-patch" );
-        IDocument* doc = docf->create( fakeUrl, ICore::self() );
-        IPatchDocument* pdoc = dynamic_cast<IPatchDocument*>( doc );
-
-        Q_ASSERT( pdoc );
-        ICore::self()->documentController()->openDocument( doc );
-        pdoc->setDiff( d.leftTexts()[l], to );
-    }
-}
-#endif
-#endif
-
 void PatchReviewPlugin::closeReview()
 {
     if( m_patch ) {
@@ -417,6 +367,7 @@ void PatchReviewPlugin::finishReview( QList<KUrl> selection ) {
 
 void PatchReviewPlugin::startReview( IPatchSource* patch, IPatchReview::ReviewMode mode ) {
     Q_UNUSED( mode );
+    emit startingNewReview();
     setPatch( patch );
     QMetaObject::invokeMethod( this, "updateReview", Qt::QueuedConnection );
 }
@@ -475,23 +426,11 @@ void PatchReviewPlugin::updateReview() {
 
     IDocument* futureActiveDoc = 0;
     //Open the diff itself
-#ifdef HAVE_KOMPARE
-    KUrl fakeUrl( m_patch->file() );
-    fakeUrl.setScheme( "kdevpatch" );
-    IDocumentFactory* docf = ICore::self()->documentController()->factory( "text/x-patch" );
-    IDocument* doc = docf->create( fakeUrl, ICore::self() );
-    IPatchDocument* pdoc = dynamic_cast<IPatchDocument*>( doc );
-
-    Q_ASSERT( pdoc );
-    futureActiveDoc = ICore::self()->documentController()->openDocument( doc );
-    //TODO: close kompare doc if available
-#else
     if ( !documents.contains( m_patch->file() ) ) {
         futureActiveDoc = ICore::self()->documentController()->openDocument( m_patch->file() );
     } else {
         futureActiveDoc = documents.take( m_patch->file() );
     }
-#endif
     if ( !futureActiveDoc || !futureActiveDoc->textDocument() ) {
         // might happen if e.g. openDocument dialog was cancelled by user
         // or under the theoretic possibility of a non-text document getting opened
@@ -499,17 +438,20 @@ void PatchReviewPlugin::updateReview() {
     }
     futureActiveDoc->textDocument()->setReadWrite( false );
     futureActiveDoc->setPrettyName( i18n( "Overview" ) );
+    
+    IDocument* buddyDoc = futureActiveDoc;
+    
     KTextEditor::ModificationInterface* modif = dynamic_cast<KTextEditor::ModificationInterface*>( futureActiveDoc->textDocument() );
     modif->setModifiedOnDiskWarning( false );
 
     if( m_modelList->modelCount() < maximumFilesToOpenDirectly ) {
         //Open all relates files
         for( int a = 0; a < m_modelList->modelCount(); ++a ) {
-            KUrl absoluteUrl( m_patch->baseDir(),  m_modelList->modelAt( a )->destination() );
+            KUrl absoluteUrl = urlForFileModel( m_modelList->modelAt( a ) );
 
             if( QFileInfo( absoluteUrl.path() ).exists() && absoluteUrl.path() != "/dev/null" )
             {
-                ICore::self()->documentController()->openDocument( absoluteUrl );
+                buddyDoc = ICore::self()->documentController()->openDocument( absoluteUrl, KTextEditor::Range::invalid(), IDocumentController::DoNotActivate, "", buddyDoc );
                 documents.remove( absoluteUrl );
 
                 seekHunk( true, absoluteUrl ); //Jump to the first changed position
@@ -554,7 +496,7 @@ void PatchReviewPlugin::setPatch( IPatchSource* patch ) {
     m_patch = patch;
 
     if( m_patch ) {
-        kDebug() << "setting new patch" << patch->name() << "with file" << patch->file();
+        kDebug() << "setting new patch" << patch->name() << "with file" << patch->file() << "basedir" << patch->baseDir();
         registerPatch( patch );
 
         connect( m_patch, SIGNAL( patchChanged() ), this, SLOT( notifyPatchChanged() ) );
@@ -618,70 +560,6 @@ void PatchReviewPlugin::exporterSelected( QAction* action ) {
     }
 }
 
-#if 0
-void PatchReviewPlugin::determineState() {
-    LocalPatchSourcePointer lpatch = m_patch;
-    if ( !lpatch ) {
-        kDebug() <<"determineState(..) could not lock patch";
-    }
-    try {
-        if ( lpatch->filename.isEmpty() )
-            throw "state can only be determined for file-patches";
-
-        KUrl fileUrl = lpatch->filename;
-
-        {
-            K3Process proc;
-            ///Try to apply, if it works, the patch is not applied
-            QString cmd =  "patch --dry-run -s -f -i " + fileUrl.toLocalFile();
-            proc << splitArgs( cmd );
-
-            kDebug() << "calling " << cmd;
-
-            if ( !proc.start( K3Process::Block ) )
-                throw "could not start process";
-
-            if ( !proc.normalExit() )
-                throw "process did not exit normally";
-
-            kDebug() << "exit-status:" << proc.exitStatus();
-
-            if ( proc.exitStatus() == 0 ) {
-//                 lpatch->state = LocalPatchSource::NotApplied;
-                return;
-            }
-        }
-
-        {
-            ///Try to revert, of it works, the patch is applied
-            K3Process proc;
-            QString cmd =  "patch --dry-run -s -f -i --reverse " + fileUrl.toLocalFile();
-            proc << splitArgs( cmd );
-
-            kDebug() << "calling " << cmd;
-
-            if ( !proc.start( K3Process::Block ) )
-                throw "could not start process";
-
-            if ( !proc.normalExit() )
-                throw "process did not exit normally";
-
-            kDebug() << "exit-status:" << proc.exitStatus();
-
-            if ( proc.exitStatus() == 0 ) {
-//                 lpatch->state = LocalPatchSource::Applied;
-                return;
-            }
-        }
-    } catch ( const QString& str ) {
-        kWarning() <<"Error:" << str;
-    } catch ( const char* str ) {
-        kWarning() << "Error:" << str;
-    }
-
-//     lpatch->state = LocalPatchSource::Unknown;
-}
-#endif
 #include "patchreview.moc"
 
 // kate: space-indent on; indent-width 2; tab-width 2; replace-tabs on
