@@ -59,6 +59,20 @@ static Qt::ConnectionType getConnectionTypeForSignalDelivery( KDevelop::ProjectM
 namespace KDevelop
 {
 
+/**
+ * Return true if @p url contains a clean path
+ */
+inline bool isValidPath(const KUrl& url)
+{
+    if (!url.isValid()) {
+        return false;
+    }
+
+    KUrl cleaned = url;
+    cleaned.cleanPath();
+    return url == cleaned;
+}
+
 QStringList removeProjectBasePath( const QStringList& fullpath, KDevelop::ProjectBaseItem* item )
 {
     QStringList result = fullpath;
@@ -409,6 +423,8 @@ void ProjectBaseItem::appendRow( ProjectBaseItem* item )
         kWarning() << "Ignoring double insertion of item" << item;
         return;
     }
+    // this is too slow... O(n) and thankfully not a problem anyways
+//     Q_ASSERT(!d->children.contains(item));
     int startrow,endrow;
     if( model() ) {
         startrow = endrow = d->children.count();
@@ -615,34 +631,38 @@ void ProjectFolderItem::propagateRename(const KUrl& newBase) const
 
 ProjectBaseItem::RenameStatus ProjectFolderItem::rename(const QString& newname)
 {
+    if (newname.contains('/'))
+        return ProjectBaseItem::InvalidNewName;
+
     //TODO: Same as ProjectFileItem, so should be shared somehow
     KUrl dest = url().upUrl();
     dest.addPath(newname);
-    if( !newname.contains('/') )
+
+    // url() may contain a trailing slash, try to ignore that
+    if (url().equals(dest, KUrl::CompareWithoutTrailingSlash)) {
+        kDebug() << "Source and destination folder equal, no rename needed";
+        return ProjectBaseItem::RenameOk;
+    }
+
+    KIO::UDSEntry entry;
+    //There exists a file with that name?
+    if( !KIO::NetAccess::stat(dest, entry, 0) )
     {
-        KIO::UDSEntry entry;
-        //There exists a file with that name?
-        if( !KIO::NetAccess::stat(dest, entry, 0) )
+        if( !project() || !project()->projectFileManager() )
         {
-            if( !project() || !project()->projectFileManager() )
-            {
-                return ProjectBaseItem::rename(newname);
-            }
-            else if( project()->projectFileManager()->renameFolder(this, dest) )
-            {
-                return ProjectBaseItem::RenameOk;
-            }
-            else
-            {
-                return ProjectBaseItem::ProjectManagerRenameFailed;
-            }
-        } else
+            return ProjectBaseItem::rename(newname);
+        }
+        else if( project()->projectFileManager()->renameFolder(this, dest) )
         {
-            return ProjectBaseItem::ExistingItemSameName;
+            return ProjectBaseItem::RenameOk;
+        }
+        else
+        {
+            return ProjectBaseItem::ProjectManagerRenameFailed;
         }
     } else
     {
-        return ProjectBaseItem::InvalidNewName;
+        return ProjectBaseItem::ExistingItemSameName;
     }
 }
 
@@ -687,56 +707,57 @@ QString ProjectBuildFolderItem::iconName() const
 ProjectFileItem::ProjectFileItem( IProject* project, const KUrl & file, ProjectBaseItem * parent )
         : ProjectBaseItem( project, file.fileName(), parent )
 {
+    Q_ASSERT(isValidPath(file));
+
     setFlags(flags() | Qt::ItemIsDragEnabled);
     setUrl( file );
-    // Need to this manually here as setUrl() is virtual and hence the above
-    // only calls the version in ProjectBaseItem and not ours
-    if( project && d_ptr->m_urlIndex ) {
-        project->addToFileSet( IndexedString::fromIndex( d_ptr->m_urlIndex ) );
-    }
 }
 
 ProjectFileItem::~ProjectFileItem()
 {
     if( project() && d_ptr->m_urlIndex ) {
-        project()->removeFromFileSet( IndexedString::fromIndex( d_ptr->m_urlIndex ) );
+        project()->removeFromFileSet( indexedUrl() );
     }
 }
 
-QString ProjectFileItem::iconName() const
+IndexedString ProjectFileItem::indexedUrl() const
 {
-    return d_ptr->iconName;
+    return IndexedString::fromIndex( d_ptr->m_urlIndex );
 }
 
 ProjectBaseItem::RenameStatus ProjectFileItem::rename(const QString& newname)
 {
+    if (newname.contains('/'))
+        return ProjectBaseItem::InvalidNewName;
+
     KUrl dest = url().upUrl();
     dest.addPath(newname);
-    if( !newname.contains('/') )
+
+    if (url() == dest) {
+        kDebug() << "Source and destination file equal, no rename needed:" << dest;
+        // it's fine to just say the rename succeeded here, std::rename has the same behavior
+        return ProjectBaseItem::RenameOk;
+    }
+
+    KIO::UDSEntry entry;
+    //There exists a file with that name?
+    if( !KIO::NetAccess::stat(dest, entry, 0) )
     {
-        KIO::UDSEntry entry;
-        //There exists a file with that name?
-        if( !KIO::NetAccess::stat(dest, entry, 0) )
+        if( !project() || !project()->projectFileManager() )
         {
-            if( !project() || !project()->projectFileManager() )
-            {
-                return ProjectBaseItem::rename(newname);
-            }
-            else if( project()->projectFileManager()->renameFile(this, dest) )
-            {
-                return ProjectBaseItem::RenameOk;
-            }
-            else
-            {
-                return ProjectBaseItem::ProjectManagerRenameFailed;
-            }
-        } else
+            return ProjectBaseItem::rename(newname);
+        }
+        else if( project()->projectFileManager()->renameFile(this, dest) )
         {
-            return ProjectBaseItem::ExistingItemSameName;
+            return ProjectBaseItem::RenameOk;
+        }
+        else
+        {
+            return ProjectBaseItem::ProjectManagerRenameFailed;
         }
     } else
     {
-        return ProjectBaseItem::InvalidNewName;
+        return ProjectBaseItem::ExistingItemSameName;
     }
 }
 
@@ -806,6 +827,18 @@ public:
 
 K_GLOBAL_STATIC(IconNameCache, s_cache);
 
+QString ProjectFileItem::iconName() const
+{
+    // think of d_ptr->iconName as mutable, possible since d_ptr is not const
+    if (d_ptr->iconName.isEmpty()) {
+        // lazy load implementation of icon lookup
+        d_ptr->iconName = s_cache->iconNameForUrl( d_ptr->m_url, d_ptr->text );
+        // we should always get *some* icon name back
+        Q_ASSERT(!d_ptr->iconName.isEmpty());
+    }
+    return d_ptr->iconName;
+}
+
 void ProjectFileItem::setUrl( const KUrl& url )
 {
     if (url == d_ptr->m_url) {
@@ -814,17 +847,18 @@ void ProjectFileItem::setUrl( const KUrl& url )
 
     if( project() && d_ptr->m_urlIndex ) {
         // remove from fileset if we are in there
-        project()->removeFromFileSet( IndexedString::fromIndex( d_ptr->m_urlIndex ) );
+        project()->removeFromFileSet( indexedUrl() );
     }
 
     ProjectBaseItem::setUrl( url );
 
     if( project() && d_ptr->m_urlIndex ) {
         // add to fileset with new url index
-        project()->addToFileSet( IndexedString::fromIndex( d_ptr->m_urlIndex ) );
+        project()->addToFileSet( indexedUrl() );
     }
 
-    d_ptr->iconName = s_cache->iconNameForUrl( url, d_ptr->text );
+    // invalidate icon name for future lazy-loaded updated
+    d_ptr->iconName.clear();
 }
 
 int ProjectFileItem::type() const
@@ -1038,10 +1072,14 @@ ProjectBaseItem* ProjectModel::takeRow( int row )
     return d->rootItem->takeRow( row );
 }
 
-bool ProjectModel::hasChildren(const QModelIndex& parent) const
+ProjectBaseItem* ProjectModel::itemAt(int row) const
 {
-    bool b = QAbstractItemModel::hasChildren(parent);
-    return b;
+    return d->rootItem->child(row);
+}
+
+QList< ProjectBaseItem* > ProjectModel::topItems() const
+{
+    return d->rootItem->children();
 }
 
 Qt::ItemFlags ProjectModel::flags(const QModelIndex& index) const
@@ -1071,7 +1109,7 @@ bool ProjectModel::setData(const QModelIndex&, const QVariant&, int)
     return false;
 }
 
-QList< ProjectBaseItem* > ProjectModel::itemsForUrl ( const KUrl& url )
+QList< ProjectBaseItem* > ProjectModel::itemsForUrl ( const KUrl& url ) const
 {
     if (!url.isValid()) {
         return QList<ProjectBaseItem*>();
@@ -1080,12 +1118,24 @@ QList< ProjectBaseItem* > ProjectModel::itemsForUrl ( const KUrl& url )
     return d->urlLookupTable.values(indexForUrl(url));
 }
 
+ProjectBaseItem* ProjectModel::itemForUrl(const IndexedString& url) const
+{
+    return d->urlLookupTable.value(url.index());
+}
+
+void ProjectVisitor::visit( ProjectModel* model )
+{
+    foreach( ProjectBaseItem* item, model->topItems() )
+    {
+        visit( item->project() );
+    }
+}
+
 void ProjectVisitor::visit ( IProject* prj )
 {
     visit( prj->projectItem() );
 }
 
-
 void ProjectVisitor::visit ( ProjectBuildFolderItem* folder )
 {
     foreach( ProjectFileItem* item, folder->fileList() )
