@@ -1,6 +1,7 @@
 /*
    Copyright 2009 David Nolden <david.nolden.kdevelop@art-master.de>
-   
+   Copyright 2012 Milian Wolff <mail@milianw.de>
+
    This library is free software; you can redistribute it and/or
    modify it under the terms of the GNU Library General Public
    License version 2 as published by the Free Software Foundation.
@@ -19,32 +20,39 @@
 #ifndef ASSISTANTPOPUP_H
 #define ASSISTANTPOPUP_H
 
-#include <QFrame>
+#include <QToolBar>
 #include <interfaces/iassistant.h>
 #include <ksharedptr.h>
 
 
-class AssistantPopup : public QFrame
+class AssistantPopup : public QToolBar
 {
     Q_OBJECT
+
 public:
     typedef KSharedPtr<AssistantPopup> Ptr;
-    AssistantPopup(QWidget* parent, KDevelop::IAssistant::Ptr assistant);
+    /**
+     * @p widget The widget below which the assistant should be shown.
+     * The current main window will be used as parent widget for the popup.
+     * This is to make use of the maximal space available and prevent any lines
+     * in e.g. the editor to be hidden by the popup.
+     */
+    AssistantPopup(QWidget* widget, const KDevelop::IAssistant::Ptr& assistant);
     KDevelop::IAssistant::Ptr assistant() const;
-    public slots:
-    void executeAction1();
-    void executeAction2();
-    void executeAction3();
-    void executeAction4();
+
+public slots:
     void executeHideAction();
-    void assistantActionsChanged();
-    private:
-        void executeAction(int number);
-        void updateActions();
-        QWidget* widgetForAction(KDevelop::IAssistantAction::Ptr action);
-        KDevelop::IAssistant::Ptr m_assistant;
-        QList<KDevelop::IAssistantAction::Ptr> m_actions;
-        QPalette m_normalPalette;
+
+private slots:
+    void updatePosition();
+
+private:
+    virtual bool eventFilter(QObject* object, QEvent* event);
+    void updateActions();
+    QWidget* widgetForAction(const KDevelop::IAssistantAction::Ptr& action, int& mnemonic);
+    KDevelop::IAssistant::Ptr m_assistant;
+    QList<KDevelop::IAssistantAction::Ptr> m_assistantActions;
+    QWidget* m_contextWidget;
 };
 
 #endif // ASSISTANTPOPUP_H
