@@ -54,6 +54,8 @@
 #include "debugcontroller.h"
 #include "kdevplatformversion.h"
 #include "workingsetcontroller.h"
+#include "testcontroller.h"
+
 #include <KMessageBox>
 
 #include <KTextEditor/Document>
@@ -68,12 +70,17 @@ void shutdownGracefully(int sig)
     if ( !handlingSignal ) {
         handlingSignal = 1;
         qDebug() << "signal " << sig << " received, shutting down gracefully";
-        QCoreApplication::instance()->quit();
-    } else {
-        // re-raise signal with default handler and trigger program termination
-        std::signal(sig, SIG_DFL);
-        std::raise(sig);
+        QCoreApplication* app = QCoreApplication::instance();
+        if (QApplication* guiApp = qobject_cast<QApplication*>(app)) {
+            guiApp->closeAllWindows();
+        }
+        app->quit();
+        return;
     }
+
+    // re-raise signal with default handler and trigger program termination
+    std::signal(sig, SIG_DFL);
+    std::raise(sig);
 }
 
 void installSignalHandler()
@@ -97,7 +104,7 @@ KAboutData aboutData()
     KAboutData aboutData( "kdevplatform", "kdevplatform", 
                           ki18n("KDevelop Platform"), KDEVPLATFORM_VERSION_STR, 
                           ki18n("Development Platform for IDE-like Applications"), 
-                          KAboutData::License_LGPL_V2, ki18n( "Copyright 2004-2009, The KDevelop developers" ), 
+                          KAboutData::License_LGPL_V2, ki18n( "Copyright 2004-2012, The KDevelop developers" ),
                           KLocalizedString(), "http://www.kdevelop.org" );
     aboutData.addAuthor( ki18n("Andreas Pakulat"), ki18n( "Architecture, VCS Support, Project Management Support, QMake Projectmanager" ), "apaku@gmx.de" );
     aboutData.addAuthor( ki18n("Alexander Dymo"), ki18n( "Architecture, Sublime UI, Ruby support" ), "adymo@kdevelop.org" );
@@ -149,23 +156,6 @@ bool CorePrivate::initialize(Core::Setup mode, QString session )
     if( !partController && !(mode & Core::NoUi))
     {
         partController = new PartController(m_core, uiController.data()->defaultMainWindow());
-
-        {
-            // check features of kate and report to user if it does not fit
-            KTextEditor::Document* doc = partController.data()->createTextPart();
-
-            if ( !qobject_cast< KTextEditor::MovingInterface* >(doc) ) {
-                KMessageBox::error(QApplication::activeWindow(),
-                                   i18n("The installed Kate version does not support the MovingInterface which is crucial for "
-                                        "KDevelop starting from version 4.2.\n\n"
-                                        "To use KDevelop with KDE SC prior to 4.6, where the SmartInterface is used instead "
-                                        "of the MovingInterface, you need KDevelop 4.1 or lower."));
-                delete doc;
-                return false;
-            }
-
-            delete doc;
-        }
     }
 
     if( !projectController )
@@ -204,7 +194,7 @@ bool CorePrivate::initialize(Core::Setup mode, QString session )
     {
         selectionController = new SelectionController(m_core);
     }
-    
+
     if( !documentationController && !(mode & Core::NoUi) )
     {
         documentationController = new DocumentationController(m_core);
@@ -214,28 +204,16 @@ bool CorePrivate::initialize(Core::Setup mode, QString session )
     {
         debugController = new DebugController(m_core);
     }
-    
-    kDebug() << "initializing ui controller";
-    
-    if( !session.isEmpty() && !SessionController::tryLockSession(session) && !(mode & Core::NoUi) )
-    {
-        QString errmsg = i18n("The session %1 is already active in another running instance. Choose another session or close the running instance.", session );
-        session = SessionController::showSessionChooserDialog(errmsg);
-        if(session.isEmpty())
-            return false;
-    }
-    
-    sessionController.data()->initialize( session );
 
-    if(!sessionController.data()->lockSession())
+    if( !testController )
     {
-        QString errmsg = i18n("Failed to lock the session %1, probably it is already active in another running instance", session );
-        if( mode & Core::NoUi ) {
-            QTextStream qerr(stderr);
-            qerr << endl << errmsg << endl;
-        } else {
-            KMessageBox::error(0, errmsg);
-        }
+        testController = new TestController(m_core);
+    }
+
+    kDebug() << "initializing ui controller";
+
+    sessionController.data()->initialize( session );
+    if( !sessionController.data()->activeSession() ) {
         return false;
     }
 
@@ -276,9 +254,27 @@ bool CorePrivate::initialize(Core::Setup mode, QString session )
         documentationController.data()->initialize();
     }
     debugController.data()->initialize();
+    testController.data()->initialize();
 
     installSignalHandler();
 
+    if (partController) {
+        // check features of kate and report to user if it does not fit
+        KTextEditor::Document* doc = partController.data()->createTextPart();
+
+        if ( !qobject_cast< KTextEditor::MovingInterface* >(doc) ) {
+            KMessageBox::error(QApplication::activeWindow(),
+                                i18n("The installed Kate version does not support the MovingInterface which is crucial for "
+                                    "KDevelop starting from version 4.2.\n\n"
+                                    "To use KDevelop with KDE SC prior to 4.6, where the SmartInterface is used instead "
+                                    "of the MovingInterface, you need KDevelop 4.1 or lower."));
+            delete doc;
+            return false;
+        }
+
+        delete doc;
+    }
+
     return true;
 }
 CorePrivate::~CorePrivate()
@@ -296,6 +292,7 @@ CorePrivate::~CorePrivate()
     delete documentationController.data();
     delete debugController.data();
     delete workingSetController.data();
+    delete testController.data();
 
     selectionController.clear();
     projectController.clear();
@@ -310,6 +307,7 @@ CorePrivate::~CorePrivate()
     documentationController.clear();
     debugController.clear();
     workingSetController.clear();
+    testController.clear();
 }
 
 
@@ -398,7 +396,9 @@ void Core::cleanup()
         d->sourceFormatterController.data()->cleanup();
         d->pluginController.data()->cleanup();
         d->sessionController.data()->cleanup();
-        
+
+        d->testController.data()->cleanup();
+
         //Disable the functionality of the language controller
         d->languageController.data()->cleanup();
     }
@@ -532,6 +532,16 @@ DebugController* Core::debugControllerInternal()
     return d->debugController.data();
 }
 
+ITestController* Core::testController()
+{
+    return d->testController.data();
+}
+
+TestController* Core::testControllerInternal()
+{
+    return d->testController.data();
+}
+
 WorkingSetController* Core::workingSetControllerInternal()
 {
     return d->workingSetController.data();
