@@ -78,6 +78,7 @@ Boston, MA 02110-1301, USA.
 #include "sessioncontroller.h"
 #include "session.h"
 #include <QApplication>
+#include <QDBusConnection>
 #include <vcs/models/projectchangesmodel.h>
 #include <vcs/widgets/vcsdiffpatchsources.h>
 #include <vcs/widgets/vcscommitdialog.h>
@@ -109,7 +110,7 @@ public:
     QPointer<ProjectChangesModel> m_changesModel;
 
     ProjectControllerPrivate( ProjectController* p )
-        : m_core(0), model(0), selectionModel(0), dialog(0), m_configuringProject(0), q(p), m_foundProjectFile(false), m_cleaningUp(false)
+        : m_core(0), model(0), selectionModel(0), dialog(0), m_configuringProject(0), q(p), buildset(0), m_foundProjectFile(false), m_cleaningUp(false)
     {
     }
 
@@ -127,10 +128,6 @@ public:
         if( !proj )
             return;
 
-        //@FIXME: compute a blacklist, based on a query for all KDevelop
-        //plugins implementing IProjectManager, removing from that the
-        //plugin that manages this project. Set this as blacklist on the
-        //dialog
         //@FIXME: Currently it is important to set a parentApp on the kcms
         //that's different from the component name of the application, else
         //the plugin will show up on all projects settings dialogs.
@@ -166,10 +163,29 @@ public:
         group.sync();
     }
 
+    // Recursively collects builder dependencies for a project.
+    static void collectBuilders( QList< IProjectBuilder* >& destination, IProjectBuilder* topBuilder, IProject* project )
+    {
+        QList< IProjectBuilder* > auxBuilders = topBuilder->additionalBuilderPlugins( project );
+        destination.append( auxBuilders );
+        foreach( IProjectBuilder* auxBuilder, auxBuilders ) {
+            collectBuilders( destination, auxBuilder, project );
+        }
+    }
+
     QStringList findPluginsForProject( IProject* project )
     {
         QList<IPlugin*> plugins = m_core->pluginController()->loadedPlugins();
         QStringList pluginnames;
+        QList< IProjectBuilder* > buildersForKcm;
+        // Important to also include the "top" builder for the project, so
+        // projects with only one such builder are kept working. Otherwise the project config
+        // dialog is empty for such cases.
+        if( IBuildSystemManager* buildSystemManager = project->buildSystemManager() ) {
+            buildersForKcm << buildSystemManager->builder();
+            collectBuilders( buildersForKcm, buildSystemManager->builder(), project );
+        }
+
         for( QList<IPlugin*>::iterator it = plugins.begin(); it != plugins.end(); ++it )
         {
             IPlugin* plugin = *it;
@@ -183,34 +199,9 @@ public:
                 continue;
             }
             IProjectBuilder* builder = plugin->extension<KDevelop::IProjectBuilder>();
-            if (builder)
+            if ( builder && !buildersForKcm.contains( builder ) )
             {
-                // hide builder plugins that are not required
-                if(!project->buildSystemManager())
-                {
-                    // current plugin is a builder yet the project cannot be build - skip
-                    continue;
-                }
-                if (builder && builder != project->buildSystemManager()->builder())
-                {
-                    const KPluginInfo builderInfo = m_core->pluginController()->pluginInfo(
-                        dynamic_cast<IPlugin*>(project->buildSystemManager()->builder()) );
-                    bool shouldBeHidden = true;
-                    // hide if this plugin does not implement an interface that is required by the builder
-                    // i.e. makebuilder should be shown for cmake projects
-                    const QStringList dependencies = builderInfo.property("X-KDevelop-IRequired").toStringList();
-                    foreach(const QString& interface, info.property("X-KDevelop-Interfaces").toStringList()) {
-                        if (dependencies.contains(interface))
-                        {
-                            shouldBeHidden = false;
-                            break;
-                        }
-                    }
-                    if (shouldBeHidden)
-                    {
-                        continue;
-                    }
-                }
+                continue;
             }
             pluginnames << info.pluginName();
         }
@@ -468,22 +459,13 @@ ProjectController::ProjectController( Core* core )
     setObjectName("ProjectController");
     d->m_core = core;
     d->model = new ProjectModel();
-    d->buildset = new ProjectBuildSetModel( this );
-    connect( this, SIGNAL(projectOpened(KDevelop::IProject*)),
-             d->buildset, SLOT(loadFromProject(KDevelop::IProject*)) );
-    connect( this, SIGNAL(projectClosing(KDevelop::IProject*)),
-             d->buildset, SLOT(saveToProject(KDevelop::IProject*)) );
-    connect( this, SIGNAL(projectClosed(KDevelop::IProject*)),
-             d->buildset, SLOT(projectClosed(KDevelop::IProject*)) );
 
-    d->selectionModel = new QItemSelectionModel(d->model);
-    if(!(Core::self()->setupFlags() & Core::NoUi)) setupActions();
-
-    loadSettings(false);
-    d->dialog = new ProjectDialogProvider(d);
-    KSettings::Dispatcher::registerComponent( KComponentData("kdevplatformproject"), 
-                                              this, 
-                                              "notifyProjectConfigurationChanged" );
+    //NOTE: this is required to be called here, such that the
+    //      actions are available when the UI controller gets
+    //      initialized *before* the project controller
+    if (Core::self()->setupFlags() != Core::NoUi) {
+        setupActions();
+    }
 }
 
 void ProjectController::setupActions()
@@ -562,9 +544,9 @@ ProjectController::~ProjectController()
 void ProjectController::cleanup()
 {
     d->m_cleaningUp = true;
-    
-    KSharedConfig::Ptr config = Core::self()->activeSession()->config();
-    KConfigGroup group = config->group( "General Options" );
+    if( buildSetModel() ) {
+        buildSetModel()->storeToSession( Core::self()->activeSession() );
+    }
 
     foreach( IProject* project, d->m_projects ) {
         closeProject( project );
@@ -573,6 +555,26 @@ void ProjectController::cleanup()
 
 void ProjectController::initialize()
 {
+    d->buildset = new ProjectBuildSetModel( this );
+    buildSetModel()->loadFromSession( Core::self()->activeSession() );
+    connect( this, SIGNAL(projectOpened(KDevelop::IProject*)),
+             d->buildset, SLOT(loadFromProject(KDevelop::IProject*)) );
+    connect( this, SIGNAL(projectClosing(KDevelop::IProject*)),
+             d->buildset, SLOT(saveToProject(KDevelop::IProject*)) );
+    connect( this, SIGNAL(projectClosed(KDevelop::IProject*)),
+             d->buildset, SLOT(projectClosed(KDevelop::IProject*)) );
+
+    d->selectionModel = new QItemSelectionModel(d->model);
+
+    loadSettings(false);
+    d->dialog = new ProjectDialogProvider(d);
+    KSettings::Dispatcher::registerComponent( KComponentData("kdevplatformproject"),
+                                              this,
+                                              "notifyProjectConfigurationChanged" );
+
+    QDBusConnection::sessionBus().registerObject( "/org/kdevelop/ProjectController",
+        this, QDBusConnection::ExportScriptableSlots );
+
     KSharedConfig::Ptr config = Core::self()->activeSession()->config();
     KConfigGroup group = config->group( "General Options" );
     KUrl::List openProjects = group.readEntry( "Open Projects", QStringList() );
@@ -777,18 +779,16 @@ void ProjectController::projectImportingFinished( IProject* project )
         KJob* parseProjectJob = new KDevelop::ParseProjectJob(project);
         ICore::self()->runController()->registerJob(parseProjectJob);
     }
-    
-    KUrl::List parseList;
+
     // Add all currently open files that belong to the project to the background-parser,
     // since more information may be available for parsing them now(Like include-paths).
     foreach(IDocument* document, Core::self()->documentController()->openDocuments())
     {
         if(!project->filesForUrl(document->url()).isEmpty())
         {
-            parseList.append(document->url());
+            Core::self()->languageController()->backgroundParser()->addDocument( IndexedString(document->url()), TopDUContext::AllDeclarationsContextsAndUses, 10 );
         }
     }
-    Core::self()->languageController()->backgroundParser()->addDocumentList( parseList, KDevelop::TopDUContext::AllDeclarationsContextsAndUses, 10 );
 }
 
 // helper method for closeProject()
@@ -962,7 +962,7 @@ QString ProjectController::prettyFilePath(KUrl url, FormattingOptions format) co
     QString prefixText = url.upUrl().pathOrUrl(KUrl::AddTrailingSlash);
     if (project) {
         if (format == FormatHtml) {
-            prefixText = "<i>" +  project->name() + "/</i>";
+            prefixText = "<i>" +  project->name() + "</i>/";
         } else {
             prefixText = project->name() + '/';
         }
@@ -1036,7 +1036,7 @@ void ProjectController::commitCurrentProject()
         if(vcs) {
             ICore::self()->documentController()->saveAllDocuments(KDevelop::IDocument::Silent);
 
-            VCSCommitDiffPatchSource* patchSource = new VCSCommitDiffPatchSource(new VCSStandardDiffUpdater(vcs, baseUrl), baseUrl, vcs);
+            VCSCommitDiffPatchSource* patchSource = new VCSCommitDiffPatchSource(new VCSStandardDiffUpdater(vcs, baseUrl));
 
             bool ret = showVcsDiff(patchSource);
 
@@ -1049,6 +1049,62 @@ void ProjectController::commitCurrentProject()
     }
 }
 
+QString ProjectController::mapSourceBuild( const QString& path, bool reverse, bool fallbackRoot ) const
+{
+    KUrl url(path);
+    IProject* sourceDirProject = 0, *buildDirProject = 0;
+    Q_FOREACH(IProject* proj, d->m_projects)
+    {
+        if(proj->folder().isParentOf(url))
+            sourceDirProject = proj;
+        if(proj->buildSystemManager())
+        {
+            KUrl buildDir = proj->buildSystemManager()->buildDirectory(proj->projectItem());
+            if(buildDir.isValid() && buildDir.isParentOf(url))
+                buildDirProject = proj;
+        }
+    }
+    
+    if(!reverse)
+    {
+        // Map-target is the build directory
+        if(sourceDirProject && sourceDirProject->buildSystemManager())
+        {
+            // We're in the source, map into the build directory
+            QString relativePath = KUrl::relativeUrl(sourceDirProject->folder(), url);
+            
+            KUrl build = sourceDirProject->buildSystemManager()->buildDirectory(sourceDirProject->projectItem());
+            build.addPath(relativePath);
+            while(!QFile::exists(build.path()))
+                build = build.upUrl();
+            return build.pathOrUrl();
+        }else if(buildDirProject && fallbackRoot)
+        {
+            // We're in the build directory, map to the build directory root
+            return buildDirProject->buildSystemManager()->buildDirectory(buildDirProject->projectItem()).pathOrUrl();
+        }
+    }else{
+        // Map-target is the source directory
+        if(buildDirProject)
+        {
+            KUrl build = buildDirProject->buildSystemManager()->buildDirectory(buildDirProject->projectItem());
+            // We're in the source, map into the build directory
+            QString relativePath = KUrl::relativeUrl(build, url);
+            
+            KUrl source = buildDirProject->folder();
+            source.addPath(relativePath);
+            while(!QFile::exists(source.path()))
+                source = source.upUrl();
+            return source.pathOrUrl();
+        }else if(sourceDirProject && fallbackRoot)
+        {
+            // We're in the source directory, map to the root
+            return sourceDirProject->folder().pathOrUrl();
+        }
+    }
+    return QString();
+}
+
 }
 
 #include "projectcontroller.moc"
