@@ -61,6 +61,7 @@ public Q_SLOTS:
 class KDEVPLATFORMSHELL_EXPORT ProjectController : public IProjectController
 {
     Q_OBJECT
+    Q_CLASSINFO( "D-Bus Interface", "org.kdevelop.ProjectController" )
     friend class Core;
     friend class CorePrivate;
     friend class ProjectPreferences;
@@ -75,6 +76,7 @@ public:
 
     virtual ProjectBuildSetModel* buildSetModel();
     virtual ProjectModel* projectModel();
+    virtual ProjectChangesModel* changesModel();
     virtual QItemSelectionModel* projectSelectionModel();
     virtual IProject* findProjectByName( const QString& name );
     IProject* findProjectForUrl( const KUrl& ) const;
@@ -94,24 +96,33 @@ public Q_SLOTS:
     virtual void openProjectForUrl( const KUrl &sourceUrl );
     virtual void fetchProject();
     virtual void openProject( const KUrl &KDev4ProjectFile = KUrl() );
+    virtual void abortOpeningProject( IProject* );
     void projectImportingFinished( IProject* );
     virtual void closeProject( IProject* );
     virtual void configureProject( IProject* );
     void eventuallyOpenProjectFile(KIO::Job*,KIO::UDSEntryList);
     void openProjectForUrlSlot(bool);
 //     void changeCurrentProject( ProjectBaseItem* );
+    void openProjects(const KUrl::List& projects);
+    void commitCurrentProject();
+
+    // Maps the given path from the source to the equivalent path within the build directory
+    // of the corresponding project. If the path is already in the build directory and fallbackRoot is true,
+    // then it is mapped to the root of the build directory.
+    // If reverse is true, maps the opposite direction, from build to source. [ Used in kdevplatform_shell_environment.sh ]
+    Q_SCRIPTABLE QString mapSourceBuild( const QString& path, bool reverse = false, bool fallbackRoot = true ) const;
 
 protected:
     virtual void loadSettings( bool projectIsLoaded );
     virtual void saveSettings( bool projectIsLoaded );
 
+    virtual void initialize();
 
 private:
     //FIXME Do not load all of this just for the project being opened...
     //void legacyLoading();
     void setupActions();
     void cleanup();
-    void initialize();
 
     // helper methods for closeProject()
     void unloadUnusedProjectPlugins(IProject* proj);
