@@ -70,7 +70,8 @@ ProjectSourcePage::~ProjectSourcePage()
 
 void ProjectSourcePage::sourceChanged(int index)
 {
-    m_locationWidget=0;
+    m_locationWidget = 0;
+    m_providerWidget = 0;
     QLayout* remoteWidgetLayout = m_ui->remoteWidget->layout();
     QLayoutItem *child;
     while ((child = remoteWidgetLayout->takeAt(0)) != 0) {
@@ -132,7 +133,6 @@ VcsJob* ProjectSourcePage::jobPerCurrent()
     } else if(m_providerWidget) {
         job=m_providerWidget->createWorkingCopy(url);
     }
-    Q_ASSERT(job);
     return job;
 }
 
@@ -149,6 +149,9 @@ void ProjectSourcePage::getVcsProject()
     }
     
     VcsJob* job=jobPerCurrent();
+    if (!job) {
+        return;
+    }
     
     m_ui->sources->setEnabled(false);
     m_ui->sourceBox->setEnabled(false);
@@ -189,7 +192,9 @@ void ProjectSourcePage::reevaluateCorrection()
     emit isCorrect(correct && m_ui->creationProgress->value() == m_ui->creationProgress->maximum());
     
     QDir d(cwd.toLocalFile());
-    bool validToCheckout=correct && (!m_locationWidget || m_locationWidget->isCorrect()); //To checkout, if it exists, it should be an empty dir
+    bool validWidget = ((m_locationWidget && m_locationWidget->isCorrect()) ||
+                       (m_providerWidget && m_providerWidget->isCorrect()));
+    bool validToCheckout = correct && validWidget; //To checkout, if it exists, it should be an empty dir
     if(correct && cwd.isLocalFile() && d.exists()) {
         validToCheckout = d.entryList(QDir::AllEntries | QDir::NoDotAndDotDot).isEmpty();
     }
@@ -200,7 +205,7 @@ void ProjectSourcePage::reevaluateCorrection()
     if(!correct)
         setStatus(i18n("You need to specify a valid or nonexistent directory to check out a project"));
     else if(!m_ui->get->isEnabled() && m_ui->workingDir->isEnabled())
-        setStatus(i18n("You need to specify a valid location for the project"));
+        setStatus(i18n("You need to specify a valid project location"));
     else
         validStatus();
 }
