@@ -27,6 +27,7 @@
 #include <QtGui/QHBoxLayout>
 #include <QtGui/QToolButton>
 #include <QtGui/QCommonStyle>
+#include <QTimer>
 
 #include <kdebug.h>
 #include <klocale.h>
@@ -65,8 +66,13 @@ class IdealToolBar : public QToolBar
         void hideWhenEmpty()
         {
             refresh();
-            connect(this, SIGNAL(visibilityChanged(bool)), SLOT(refresh()));
-            connect(m_buttons, SIGNAL(emptyChanged()), SLOT(refresh()));
+            
+            QTimer* t = new QTimer(this);
+            t->setInterval(100);
+            t->setSingleShot(true);
+            connect(t, SIGNAL(timeout()), SLOT(refresh()));
+            connect(this, SIGNAL(visibilityChanged(bool)), t, SLOT(start()));
+            connect(m_buttons, SIGNAL(emptyChanged()), t, SLOT(start()));
         }
 
     public slots:
@@ -133,9 +139,11 @@ MainWindowPrivate::MainWindowPrivate(MainWindow *w, Controller* controller)
     idealController = new IdealController(m_mainWindow);
 
     IdealToolBar* leftToolBar = new IdealToolBar(i18n("Left Button Bar"), idealController->leftBarWidget, m_mainWindow);
+    leftToolBar->hideWhenEmpty();
     m_mainWindow->addToolBar(Qt::LeftToolBarArea, leftToolBar);
 
     IdealToolBar* rightToolBar = new IdealToolBar(i18n("Right Button Bar"), idealController->rightBarWidget, m_mainWindow);
+    rightToolBar->hideWhenEmpty();
     m_mainWindow->addToolBar(Qt::RightToolBarArea, rightToolBar);
 
     IdealToolBar* bottomToolBar = new IdealToolBar(i18n("Bottom Button Bar"), idealController->bottomBarWidget, m_mainWindow);
@@ -283,6 +291,7 @@ Area::WalkerMode MainWindowPrivate::ViewCreator::operator() (AreaIndex *index)
         {
             // After unsplitting, we might have to remove old splitters
             QWidget* widget = splitter->widget(0);
+            kDebug() << "deleting" << widget;
             widget->setParent(0);
             delete widget;
         }
@@ -305,9 +314,13 @@ Area::WalkerMode MainWindowPrivate::ViewCreator::operator() (AreaIndex *index)
         container->show();
 
         int position = 0;
+        bool hadActiveView = false;
+        Sublime::View* activeView = d->activeView;
+        
         foreach (View *view, index->views())
         {
             QWidget *widget = view->widget(container);
+            
             if (widget)
             {
                 if(!container->hasWidget(widget))
@@ -316,7 +329,11 @@ Area::WalkerMode MainWindowPrivate::ViewCreator::operator() (AreaIndex *index)
                     d->viewContainers[view] = container;
                     d->widgetToView[widget] = view;
                 }
-                if(d->activeView == view)
+                if(activeView == view)
+                {
+                    hadActiveView = true;
+                    container->setCurrentWidget(widget);
+                }else if(topViews.contains(view) && !hadActiveView)
                     container->setCurrentWidget(widget);
             }
             position++;
@@ -325,9 +342,9 @@ Area::WalkerMode MainWindowPrivate::ViewCreator::operator() (AreaIndex *index)
     return Area::ContinueWalker;
 }
 
-void MainWindowPrivate::reconstructViews()
+void MainWindowPrivate::reconstructViews(QList<View*> topViews)
 {
-    ViewCreator viewCreator(this);
+    ViewCreator viewCreator(this, topViews);
     area->walkViews(viewCreator, area->rootIndex());
     setBackgroundVisible(area->views().isEmpty());
 }
