@@ -20,6 +20,7 @@
 #define SUBLIMEVIEW_H
 
 #include <QtCore/QObject>
+#include <QtCore/QMetaType>
 
 #include "sublimeexport.h"
 
@@ -105,5 +106,7 @@ private:
 
 }
 
+Q_DECLARE_METATYPE(Sublime::View*)
+
 #endif
 
